/* ----------------------------------------------------------------------------
 *         ATMEL Microcontroller Software Support  -  ROUSSET  -
 * ----------------------------------------------------------------------------
 * Copyright (c) 2007, Stelian Pop <stelian.pop@leadtechdesign.com>
 * Copyright (c) 2007 Lead Tech Design <www.leadtechdesign.com>
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaiimer below.
 *
 * Atmel's name may not be used to endorse or promote products derived from
 * this software without specific prior written permission.
 *
 * DISCLAIMER: THIS SOFTWARE IS PROVIDED BY ATMEL "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT ARE
 * DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * ----------------------------------------------------------------------------
 * File Name           : sdramc.c
 * Object              :
 * Creation            :
 *-----------------------------------------------------------------------------
 */
#include "../include/part.h"
#include "../include/main.h"
#include "../include/ddramc.h"

#ifdef CONFIG_DDR2

/* Write DDRC register */
static void write_ddramc(unsigned int address, unsigned int offset,
                         const unsigned int value)
{
    writel(value, (address + offset));
}

/* Read DDRC registers */
static unsigned int read_ddramc(unsigned int address, unsigned int offset)
{
    return readl((address + offset));
}

void Wait(unsigned int count)
{
    volatile unsigned int i;

#ifdef WINCE
    volatile unsigned int j = 0;
#endif

    for (i = 0; i < count; i++) {
#ifdef WINCE
        j++;
#else
        asm("    nop");
#endif
    }
}

void sdram_init(unsigned char busWidth)
{
//    AT91PS_HDDRSDRC2 pDdrc = AT91C_BASE_DDR2CP1;
    volatile unsigned int *pDdr = (unsigned int *) AT91C_EBI_SDRAM;
    int i;
    volatile unsigned int cr = 0;
    unsigned short ddrc_dbw = 0;

    // Configure EBI
    write_ddramc(AT91C_BASE_CCFG, CCFG_EBICSA, (read_ddramc(AT91C_BASE_CCFG, CCFG_EBICSA) | AT91C_EBI_CS1A_SDRAMC));
    
    switch (busWidth) {
        case 16:
        default:
            ddrc_dbw = AT91C_DDRC2_DBW_16_BITS;
            break;

        case 32:
            ddrc_dbw = AT91C_DDRC2_DBW_32_BITS;
            break;

    }
    

    // Enable DDR2 clock x2 in PMC
    write_ddramc(AT91C_BASE_PMC, PMC_SCER, AT91C_PMC_DDR);

    // Disable anticipated read
    write_ddramc(AT91C_BASE_DDR2CP1, HDDRSDRC2_HS, (read_ddramc(AT91C_BASE_DDR2CP1, HDDRSDRC2_HS) | AT91C_DDRC2_NO_ANT));

  
    //Configure DDR_MICRON_MT47H64M8.
    // Step 1: Program the memory device type
    write_ddramc(AT91C_BASE_DDR2CP1, HDDRSDRC2_MDR, ddrc_dbw   |
                                AT91C_DDRC2_MD_SDR_SDRAM);     // DDR2
    
    // Step 2:                            
    // 1. Program the features of DDR2-SDRAM device into the Configuration Register.                    
    // 2. Program the features of DDR2-SDRAM device into the Timing Register HDDRSDRC2_T0PR.            
    // 3. Program the features of DDR2-SDRAM device into the Timing Register HDDRSDRC2_T1PR.            
    // 4. Program the features of DDR2-SDRAM device into the Timing Register HDDRSDRC2_T2PR.            

    write_ddramc(AT91C_BASE_DDR2CP1, HDDRSDRC2_CR, AT91C_DDRC2_NC_DDR10_SDR9  |     // 10 column bits (1K)
                               AT91C_DDRC2_NR_13          |     // 14 row bits    (8K)
                               AT91C_DDRC2_CAS_2          |     // CAS Latency 3
                               AT91C_DDRC2_DLL_RESET_DISABLED
                               ); // DLL not reset

    // assume timings for 7.5ns min clock period
    write_ddramc(AT91C_BASE_DDR2CP1, HDDRSDRC2_T0PR, AT91C_DDRC2_TRAS_5       |     //  6 * 7.5 = 45 ns
                                 AT91C_DDRC2_TRCD_2       |     //  2 * 7.5 = 15 ns
                                 AT91C_DDRC2_TWR_2        |     //  2 * 7.5 = 15 ns
                                 AT91C_DDRC2_TRC_7        |     //  8 * 7.5 = 75 ns
                                 AT91C_DDRC2_TRP_2        |     //  2 * 7.5 = 15 ns
                                 AT91C_DDRC2_TRRD_1       |     //  1 * 7.5 = 7.5 ns
                                 AT91C_DDRC2_TWTR_1       |     //  1 clock cycle
                                 AT91C_DDRC2_TMRD_2);           //  2 clock cycles

    write_ddramc(AT91C_BASE_DDR2CP1, HDDRSDRC2_T1PR, AT91C_DDRC2_TXP_2  |           //  2 * 7.5 = 15 ns
                                 200 << 16          |           // 200 clock cycles, TXSRD: Exit self refresh delay to Read command
                                 16 << 8            |           // 16 * 7.5 = 120 ns TXSNR: Exit self refresh delay to non read command
                                 AT91C_DDRC2_TRFC_14 << 0);     // 14 * 7.5 = 105 ns (must be 105 ns for 512M DDR)

    write_ddramc(AT91C_BASE_DDR2CP1, HDDRSDRC2_T2PR, AT91C_DDRC2_TRTP_1   |         //  1 * 7.5 = 7.5 ns
                                 AT91C_DDRC2_TRPA_0   |         
                                 AT91C_DDRC2_TXARDS_7 |         //  7 clock cycles
                                 AT91C_DDRC2_TXARD_2);          //  2 clock cycles


    // Initialization Step 3 (must wait 200 us) (6 core cycles per iteration, core is at 396MHz: min 13200 loops)
    for (i = 0; i < 13300; i++) {
        asm("    nop");
    }
    // Step 4:  An NOP command is issued to the DDR2-SDRAM 
    // NOP command -> allow to enable cke
    write_ddramc(AT91C_BASE_DDR2CP1, HDDRSDRC2_MR, AT91C_DDRC2_MODE_NOP_CMD);
    *pDdr = 0;
    
    // wait 400 ns min
    for (i = 0; i < 100; i++) {
        asm("    nop");
    }

    // Step 5: An all banks precharge command is issued to the DDR2-SDRAM.
    write_ddramc(AT91C_BASE_DDR2CP1, HDDRSDRC2_MR, AT91C_DDRC2_MODE_PRCGALL_CMD);
    *pDdr = 0;
    
    // wait 400 ns min
    for (i = 0; i < 100; i++) {
        asm("    nop");
    }

    pDdr++;

    write_ddramc(AT91C_BASE_DDR2CP1, HDDRSDRC2_MR, AT91C_DDRC2_MODE_RFSH_CMD);
    *pDdr = 1;
    pDdr++;
    write_ddramc(AT91C_BASE_DDR2CP1, HDDRSDRC2_MR, AT91C_DDRC2_MODE_RFSH_CMD);
    *pDdr = 2;
    pDdr++;
    write_ddramc(AT91C_BASE_DDR2CP1, HDDRSDRC2_MR, AT91C_DDRC2_MODE_RFSH_CMD);
    *pDdr = 3;
    pDdr++;
    write_ddramc(AT91C_BASE_DDR2CP1, HDDRSDRC2_MR, AT91C_DDRC2_MODE_RFSH_CMD);
    *pDdr = 4;
    pDdr++;
    write_ddramc(AT91C_BASE_DDR2CP1, HDDRSDRC2_MR, AT91C_DDRC2_MODE_RFSH_CMD);
    *pDdr = 5;
    pDdr++;
    write_ddramc(AT91C_BASE_DDR2CP1, HDDRSDRC2_MR, AT91C_DDRC2_MODE_RFSH_CMD);
    *pDdr = 6;
    pDdr++;
    write_ddramc(AT91C_BASE_DDR2CP1, HDDRSDRC2_MR, AT91C_DDRC2_MODE_RFSH_CMD);
    *pDdr = 7;
    pDdr++;
    write_ddramc(AT91C_BASE_DDR2CP1, HDDRSDRC2_MR, AT91C_DDRC2_MODE_RFSH_CMD);
    *pDdr = 8;
    pDdr++;

    write_ddramc(AT91C_BASE_DDR2CP1, HDDRSDRC2_MR, AT91C_DDRC2_MODE_LMR_CMD);
//    *pDdr = 0xcafedede;
    *((unsigned int *)((unsigned char *)pDdr + 0x2000000)) = 0xcafedede;
    pDdr++;

    write_ddramc(AT91C_BASE_DDR2CP1, HDDRSDRC2_RTR, (BOARD_MCK * 7) / 1000000);

    write_ddramc(AT91C_BASE_DDR2CP1, HDDRSDRC2_MR, AT91C_DDRC2_MODE_NORMAL_CMD);
    *pDdr = 0;
}

//*----------------------------------------------------------------------------
//* \fn    sdram_init
//* \brief Initialize the SDDRC Controller
//*----------------------------------------------------------------------------
int ddram_init(unsigned int ddram_controller_address,
               unsigned int ddram_address, struct SDdramConfig *ddram_config)
{
    unsigned int cr = 0;

    // Step 1: Program the memory device type
    // Configure the DDR controller
    write_ddramc(ddram_controller_address, HDDRSDRC2_MDR,
                 ddram_config->ddramc_mdr);

    // Program the DDR Controller
    write_ddramc(ddram_controller_address, HDDRSDRC2_CR,
                 ddram_config->ddramc_cr);

    // assume timings for 7.5ns min clock period
    write_ddramc(ddram_controller_address, HDDRSDRC2_T0PR,
                 ddram_config->ddramc_t0pr);

    // pSDDRC->HDDRSDRC2_T1PR 
    write_ddramc(ddram_controller_address, HDDRSDRC2_T1PR,
                 ddram_config->ddramc_t1pr);

    // pSDDRC->HDDRSDRC2_T2PR 
    write_ddramc(ddram_controller_address, HDDRSDRC2_T2PR,
                 ddram_config->ddramc_t2pr);

    // Initialization Step 3: NOP command -> allow to enable clk
    write_ddramc(ddram_controller_address, HDDRSDRC2_MR,
                 AT91C_DDRC2_MODE_NOP_CMD);
    *((unsigned volatile int *)ddram_address) = 0;

    // Initialization Step 3 (must wait 200 us) (6 core cycles per iteration, core is at 396MHz: min 13200 loops)
    Wait(13300);

    // Step 4:  An NOP command is issued to the DDR2-SDRAM 
    // NOP command -> allow to enable cke
    write_ddramc(ddram_controller_address, HDDRSDRC2_MR,
                 AT91C_DDRC2_MODE_NOP_CMD);
    *((unsigned volatile int *)ddram_address) = 0;

    // wait 400 ns min
    Wait(100);

    // Initialization Step 5: Set All Bank Precharge
    write_ddramc(ddram_controller_address, HDDRSDRC2_MR,
                 AT91C_DDRC2_MODE_PRCGALL_CMD);
    *((unsigned volatile int *)ddram_address) = 0;

    // wait 400 ns min
    Wait(100);

    // Initialization Step 6: Set EMR operation (EMRS2)
    write_ddramc(ddram_controller_address, HDDRSDRC2_MR,
                 AT91C_DDRC2_MODE_EXT_LMR_CMD);
//    *((unsigned int *)(ddram_address + 0x4000000)) = 0;
    *((unsigned int *)(ddram_address + 0x2000000)) = 0;

    // wait 2 cycles min
    Wait(100);

    // Initialization Step 7: Set EMR operation (EMRS3)
    write_ddramc(ddram_controller_address, HDDRSDRC2_MR,
                 AT91C_DDRC2_MODE_EXT_LMR_CMD);
//    *((unsigned int *)(ddram_address + 0x6000000)) = 0;
    *((unsigned int *)(ddram_address + 0x3000000)) = 0;

    // wait 2 cycles min
    Wait(100);

    // Initialization Step 8: Set EMR operation (EMRS1)
    write_ddramc(ddram_controller_address, HDDRSDRC2_MR,
                 AT91C_DDRC2_MODE_EXT_LMR_CMD);
//    *((unsigned int *)(ddram_address + 0x2000000)) = 0;
    *((unsigned int *)(ddram_address + 0x0)) = 0;

    // wait 200 cycles min
    Wait(10000);

    // Initialization Step 9: enable DLL reset
    cr = read_ddramc(ddram_controller_address, HDDRSDRC2_CR);
    write_ddramc(ddram_controller_address, HDDRSDRC2_CR,
                 cr | AT91C_DDRC2_DLL_RESET_ENABLED);

    // Initialization Step 10: reset DLL
    write_ddramc(ddram_controller_address, HDDRSDRC2_MR,
                 AT91C_DDRC2_MODE_EXT_LMR_CMD);
    *(((unsigned volatile int *)ddram_address)) = 0;

    // wait 2 cycles min
    Wait(100);

    // Initialization Step 11: Set All Bank Precharge
    write_ddramc(ddram_controller_address, HDDRSDRC2_MR,
                 AT91C_DDRC2_MODE_PRCGALL_CMD);
    *(((unsigned volatile int *)ddram_address)) = 0;

    // wait 400 ns min
    Wait(100);

    // Initialization Step 12: Two auto-refresh (CBR) cycles are provided. Program the auto refresh command (CBR) into the Mode Register.
    write_ddramc(ddram_controller_address, HDDRSDRC2_MR,
                 AT91C_DDRC2_MODE_RFSH_CMD);
    *(((unsigned volatile int *)ddram_address)) = 0;

    // wait 10 cycles min
    Wait(100);

    // Set 2nd CBR
    write_ddramc(ddram_controller_address, HDDRSDRC2_MR,
                 AT91C_DDRC2_MODE_RFSH_CMD);
    *(((unsigned volatile int *)ddram_address)) = 0;

    // wait 10 cycles min
    Wait(100);

    // Initialization Step 13: Program DLL field into the Configuration Register to low(Disable DLL reset).
    cr = read_ddramc(ddram_controller_address, HDDRSDRC2_CR);
    write_ddramc(ddram_controller_address, HDDRSDRC2_CR,
                 cr & (~AT91C_DDRC2_DLL_RESET_ENABLED));

    // Initialization Step 14: A Mode Register set (MRS) cycle is issued to program the parameters of the DDR2-SDRAM devices.
    write_ddramc(ddram_controller_address, HDDRSDRC2_MR,
                 AT91C_DDRC2_MODE_LMR_CMD);
    *(((unsigned volatile int *)ddram_address)) = 0;

    // Step 15: Program OCD field into the Configuration Register to high (OCD calibration default).
    cr = read_ddramc(ddram_controller_address, HDDRSDRC2_CR);
    write_ddramc(ddram_controller_address, HDDRSDRC2_CR,
                 cr | AT91C_DDRC2_OCD_DEFAULT);

    // Step 16: An Extended Mode Register set (EMRS1) cycle is issued to OCD default value.
    write_ddramc(ddram_controller_address, HDDRSDRC2_MR,
                 AT91C_DDRC2_MODE_EXT_LMR_CMD);
//    *(((unsigned int *)(ddram_address + 0x2000000))) = 0;
    *(((unsigned int *)(ddram_address + 0x1000000))) = 0;

    // wait 2 cycles min
    Wait(100);

    // Step 17: Program OCD field into the Configuration Register to low (OCD calibration mode exit).
    cr = read_ddramc(ddram_controller_address, HDDRSDRC2_CR);
    write_ddramc(ddram_controller_address, HDDRSDRC2_CR,
                 cr & (~AT91C_DDRC2_OCD_EXIT));

    // Step 18: An Extended Mode Register set (EMRS1) cycle is issued to enable OCD exit.
    write_ddramc(ddram_controller_address, HDDRSDRC2_MR,
                 AT91C_DDRC2_MODE_EXT_LMR_CMD);
//    *(((unsigned int *)(ddram_address + 0x6000000))) = 0;
    *(((unsigned int *)(ddram_address + 0x3000000))) = 0;

    // wait 2 cycles min
    Wait(100);

    // Step 19,20: A mode Normal command is provided. Program the Normal mode into Mode Register.
    write_ddramc(ddram_controller_address, HDDRSDRC2_MR,
                 AT91C_DDRC2_MODE_NORMAL_CMD);
    *(((unsigned volatile int *)ddram_address)) = 0;

    // Step 21: Write the refresh rate into the count field in the Refresh Timer register. The DDR2-SDRAM device requires a
    // refresh every 15.625 Is or 7.81 s. With a 100MHz frequency, the refresh timer count register must to be set with
    // (15.625 /100 MHz) = 1562 i.e. 0x061A or (7.81 /100MHz) = 781 i.e. 0x030d.

    // Set Refresh timer
    write_ddramc(ddram_controller_address, HDDRSDRC2_RTR,
                 ddram_config->ddramc_rtr);

    // OK now we are ready to work on the DDRSDR

    // wait for end of calibration
    Wait(500);

    return 0;
}

#endif                          /* CONFIG_DDR2 */
